######################  
# Copyright  2000 Portal Scripts.
# All Rights Reserved.  
#                       
# The original developer of this software module and his/her company, the subsequent
# editors and their companies, have no liability for use of this
# software module or modifications thereof in an implementation.
#                       
# Permission to use, copy, and modify this software and its documentation on
# the server of the customer is hereby granted, provided that the above copyright
# notice, this paragraph and the following two paragraphs appear in all
# copies.               
#                       
# This software program and documentation are copyrighted by Portal Scripts. 
# The software program and documentation are supplied "as is", without any 
# accompanying services. Portal Scripts does not 
# warrant that the operation of the program will be
# uninterrupted or error-free.
#                       
# IN NO EVENT SHALL PORTAL SCRIPTS BE LIABLE TO ANY PARTY FOR
# DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
# LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION.
#                       
######################  

use strict;
use vars qw(%months_hash);

%months_hash = qw(
	1	January
	2	February
	3	March
	4	April
	5	May
	6	June
	7	July
	8	August
	9	September
	10	October
	11	November
	12	December
);

sub Parse_Form {
	my @pairs;
	my @getpairs;
	my $buffer;
	my $pair;
	my ($key, $value);
	my %formdata;
	if ($ENV{'REQUEST_METHOD'} eq 'GET') {
		@pairs = split(/&/, $ENV{'QUERY_STRING'});
	} elsif ($ENV{'REQUEST_METHOD'} eq 'POST') {
		read (STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
		@pairs = split(/&/, $buffer);
		
		if ($ENV{'QUERY_STRING'}) {
			@getpairs =split(/&/, $ENV{'QUERY_STRING'});
			push(@pairs,@getpairs);
			}
	} else {
		print "Content-type: text/html\n\n";
		print "<P>Use Post or Get";
	}

	foreach $pair (@pairs) {
		($key, $value) = split (/=/, $pair);
		$key =~ tr/+/ /;
		$key =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
		$value =~ tr/+/ /;
		$value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
	
		$value =~s/<!--(.|\n)*-->//g;
	
		if ($formdata{$key}) {
			$formdata{$key} .= ", $value";
		} else {
			$formdata{$key} = $value;
		}
	}
	return %formdata;
}

sub get_url {
	my $cgi_url = $ENV{'SCRIPT_NAME'};	
	$cgi_url =~ s#/([^/]*)$##;
	my $script_name = $1;
	return ($cgi_url, $script_name);
}

sub read_config {
	open(LIST, "system.conf");
	flock(LIST, 2);
	my @configs = <LIST>;
	flock(LIST, 8);
	close(LIST);
	chomp(@configs);

	for my $conf (@configs){
		$conf =~ s#\s*^##;
	}
	@configs = grep{length} @configs;
	my %CONFIG;
	for (@configs){
		my ($ckey, $cvalue) = split /\s*==\s*/, $_;
		$ckey =~ s#^\s+##;
		$cvalue =~ s#\s+$##;
		$CONFIG{$ckey} = $cvalue;
	}
	return %CONFIG;
}

sub set_cookie {
	my ($type, $cookie) = @_;
	print "Set-Cookie:$type=$cookie\n";
}

sub get_cookie {
	my %cookies;
	my ($name, $value);
	if($ENV{'HTTP_COOKIE'}){
		my @cookies = split(/;/, $ENV{'HTTP_COOKIE'});
		for (@cookies){
			($name,$value) = split(/=/,$_);
			$cookies{$name}=$value;
		}
	}
	return %cookies;
}




sub get_header {
	my ($remote_host, $path, $remote_port) = @_;
	use IO::Socket;

	no strict;
	my $socket = IO::Socket::INET->new(PeerAddr => $remote_host,
									PeerPort => $remote_port,
									Proto => "tcp",
									Type => SOCK_STREAM,
									Timeout => 5
									)
										
						or disp_message("Couldn't connect to $remote_host:$remote_port : $@");
	use strict;	
		
	print $socket join("\015\012" =>
                    "HEAD $path HTTP/1.0",
                    "Host: $remote_host",
                    "", "");
	
	my $head = "";
	while(my $content = <$socket>){
		$head .=  $content;
	}
	return $head;

}

sub get_htmlcode {
	my ($remote_host, $path, $remote_port) = @_;
	use IO::Socket;

	no strict;
	my $socket = IO::Socket::INET->new(PeerAddr => $remote_host,
									PeerPort => $remote_port,
									Proto => "tcp",
									Type => SOCK_STREAM,
									Timeout => 5
									)
										
						or disp_message("Couldn't connect to $remote_host:$remote_port : $@");
	use strict;	
		
	print $socket join("\015\012" =>
                    "HEAD $path HTTP/1.0",
                    "Host: $remote_host",
                    "", "");
	
	my $head = "";
	my $content = <$socket>;
	$content =~ m#(\d\d\d)#;
	my $code = $1;
	return $code;
}


sub get_url_content {
	my ($remote_host, $path, $remote_port) = @_;

	use IO::Socket;

	no strict;
	my $socket = IO::Socket::INET->new(PeerAddr => $remote_host,
									PeerPort => $remote_port,
									Proto => "tcp",
									Type => SOCK_STREAM,
									Timeout => 5
									)
										
						or disp_message("Couldn't connect to $remote_host:$remote_port : $@");
	use strict;	
		
   print $socket join("\015\012" =>
                    "GET $path HTTP/1.0",
                    "Host: $remote_host",
                    "", "");
#	print $socket "GET $path HTTP/1.0\r\n\r\n";
	
	my $check = 0;
	my $html = "";
	my $first_line = <$socket>;
	my $header = $first_line;
	my ($code, $location);
	if($first_line =~ m#HTTP/1\.\d (\d\d\d)#i){
		$code = $1;
	}
	
	while(my $content = <$socket>){
		
		if($content eq "\r\n"){
			$check++;
			next;	
		}
		if($check == 0){
			$header .= $content;
		}
		else{
			$html .= $content;		
		}
	}

	if($code == 302 or $code == 301 ){
		if($header =~ m#Location:\s*(.*)#i){ 
			$location = $1;
		}	
	}
	
	return ($html, $code, $header, $location);	

}


sub url_parser {
	my ($url) = @_;
	$url =~ m#^http://([^/]+)(:\d+)?(/.*)?$#;
	my $remote_host = $1;
	my $remote_port = $2;
	my $path = $3;
	unless(defined($remote_port)){
		$remote_port = 80;	
	}
	unless($path =~ m#^/#){
		$path = "/" . $path;
	}

	return ($remote_host, $path, $remote_port);
}


sub redirect_url {
	my ($url, $text) = @_;	
	print "Content-type: text/html\n\n";
	print qq!


		<html>
		<head>
		<META http-equiv="refresh" content="1;
		url=$url">
		</head>
		<b> $text </b>
		<P>
		Please wait while you are being redirected.
		</p>
		
		</html> 

	!;
	
}

sub simplify_cat {
	my ($cat) = @_;
	my $pure_fav;
	my @pure_favs = grep{length} split(/\//, $cat);			
	if(scalar(@pure_favs) > 1){
		$pure_fav = $pure_favs[-2] . ": " . $pure_favs[-1]; 
	}
	else{
		$pure_fav = $pure_favs[-1]; 
	}
	return $pure_fav;
}



sub disp_message {
	my ($message) = @_;
	print "Content-type: text/html\n\n";

	print qq!

		<html>

		<head>
		<title>Message</title>
		</head>

		<body>

		<table border="1" width="70%">
		  <tr>
		    <td width="100%">
		      <p align="center"><b><font color="#FF0000">$message</font></b></td>
		  </tr>
		</table>

		</body>

		</html>



	!;
	exit;
}


sub print_screen {
	my ($message) = @_;
	print "Content-type: text/html\n\n";

	print qq!


		<table border="1" width="70%">
		  <tr>
		    <td width="100%">
$message
		      </td>
		  </tr>
		</table>

	!;

}





sub spider_url {
	my ($url) = @_;
	my ($remote_host, $path, $remote_port) = url_parser($url);
	
	my ($html, $code, $header, $location) = get_url_content($remote_host, $path, $remote_port);
	while(defined($location)){
		($remote_host, $path, $remote_port) = url_parser($location);
		($html, $code, $header, $location) = get_url_content($remote_host, $path, $remote_port);
	}
	return ($html, $code, $header);
}


sub uri_decode {
	my ($text) = @_;
	$text =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
	return $text;

}

sub uri_encode {
	my ($text) = @_;
	
	my %escapes;
	for (0..255) {
	    $escapes{chr($_)} = sprintf("%%%02X", $_);
	}
	$text =~ s/(.)/$escapes{$1}/g;

	return $text;

}


sub uri_escape{
    my($text) = @_;
    return undef unless defined $text;
	my %escapes;
	for (0..255) {
	    $escapes{chr($_)} = sprintf("%%%02X", $_);
	}
	# Default unsafe characters. (RFC 2396 ^uric)
	$text =~ s/([^;\/?:@&=+\$,A-Za-z0-9\-_.!~*'()])/$escapes{$1}/g;
    $text;
}


sub uri_unescape{
	my ($text) = @_;
	return uri_decode($text);
}



1;

